import numpy as np

def bezier(X, lengths):
    smoothed_X = []
    for l in lengths:
        smoothed_chrom = []
        for i_locus in range(l):
            if i_locus == 0:
                x = X[i_locus]


        smoothed_X.append(smoothed_chrom)
    return smoothed_X



from iced.filter import filter_low_counts, filter_high_counts
from iced.datasets import load_sample_yeast
import numpy as np
from numpy.testing import assert_array_equal
from scipy import sparse


def test_filter_low_counts():
    X = np.ones((100, 100))
    X[0, :] = 0
    X[:, 0] = 0

    X_filtered_true = X.copy()
    X_filtered_true[X == 0] = np.nan
    X_filtered = filter_low_counts(X)
    assert_array_equal(X_filtered, X_filtered_true)

    lengths = np.array([40, 60])
    X_filtered = filter_low_counts(X, lengths=lengths)
    assert_array_equal(X_filtered, X_filtered_true)

    X_filtered = filter_low_counts(X, sparsity=False)
    assert_array_equal(X_filtered, X_filtered_true)


def test_filter_low_counts_with_zeros():
    X = 10 * np.ones((100, 100))
    X[0, :] = 0
    X[:, 0] = 0
    X[1, :] = 1
    X[:, 1] = 1

    X_filtered_true = X.copy()
    X_filtered_true[X != 10] = np.nan
    X_filtered = filter_low_counts(X, remove_all_zeros_loci=True,
                                   sparsity=False)
    assert_array_equal(X_filtered, X_filtered_true)


def test_sparse_filter_low_counts():
    X = 10 * np.ones((100, 100))
    X[0, :] = 1
    X[:, 0] = 1
    X_filtered_dense = X.copy()
    X_filtered_dense[0] = 0
    X_filtered_dense[:, 0] = 0
    return
    # this is not implemented yet
    X_filtered_sparse_csr = filter_low_counts(sparse.csr_matrix(X),
                                              sparsity=False)
    X_filtered_sparse_coo = filter_low_counts(sparse.coo_matrix(X))

    assert_array_equal(X_filtered_dense,
                       np.array(X_filtered_sparse_csr.todense()))
    assert_array_equal(X_filtered_dense,
                       np.array(X_filtered_sparse_coo.todense()))


def test_sparse_filter_low_counts_real_data():
    counts, lengths = load_sample_yeast()
    counts_sparse = sparse.csr_matrix(counts)
    counts_dense = filter_low_counts(counts, sparsity=False, percentage=0.1)
    counts_sparse = filter_low_counts(counts_sparse, sparsity=False,
                                      percentage=0.1)
    counts_dense[np.isnan(counts_dense)] = 0
    assert_array_equal(counts_dense, counts_sparse.toarray())

    triu_counts_sparse = sparse.csr_matrix(np.triu(counts))
    triu_counts_sparse = filter_low_counts(triu_counts_sparse, sparsity=False,
                                           percentage=0.1)
    assert_array_equal(np.triu(counts_dense), triu_counts_sparse.toarray())


def test_filter_high_counts():
    X = np.ones((100, 100))
    X *= np.arange(len(X))
    X_filtered_true = X.copy()
    X_filtered_true[-1] = np.nan
    X_filtered_true[:, -1] = np.nan
    X_filtered = filter_high_counts(X)
    assert_array_equal(X_filtered, X_filtered_true)


def test_filter_high_couts_sparse():
    X = np.ones((100, 100))
    X *= np.arange(len(X))
    X_filtered_dense = filter_high_counts(X)

    X_filtered_sparse_csr = filter_high_counts(sparse.csr_matrix(X))
    X_filtered_sparse_coo = filter_high_counts(sparse.coo_matrix(X))

    assert_array_equal(X_filtered_dense,
                       np.array(X_filtered_sparse_csr.todense()))
    assert_array_equal(X_filtered_dense,
                       np.array(X_filtered_sparse_coo.todense()))



import numpy as np
from scipy import sparse
from sklearn.metrics import euclidean_distances
from numpy.testing import assert_array_almost_equal
from pastis.optimization import negative_binomial_structure


def test_negative_binomial_obj_sparse():
    n = 10
    random_state = np.random.RandomState(42)
    X = random_state.rand(n, 3)
    dis = euclidean_distances(X)
    alpha, beta = -3, 1

    counts = beta * dis ** alpha
    counts = np.triu(counts)
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(counts)

    obj = negative_binomial_structure.negative_binomial_obj(
        X, counts, alpha=alpha, beta=beta)

    obj_ = negative_binomial_structure.negative_binomial_obj(
        random_state.rand(*X.shape),
        counts, alpha=alpha, beta=beta)
    assert(obj < obj_)


def test_negative_binomial_obj_sparse_dispersion_biased():
    n = 10
    random_state = np.random.RandomState(42)
    X = random_state.rand(n, 3)
    dis = euclidean_distances(X)
    alpha, beta = -3, 1

    counts = beta * dis ** alpha

    return True
    from minorswing import dispersion
    mean, variance = dispersion.compute_mean_variance(
        counts**2,
        np.array([counts.shape[0]]))
    mean, variance = mean[:-1], variance[:-1]
    d = dispersion.Dispersion()
    d.fit(mean, variance)

    counts = np.triu(counts)
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(counts)

    obj = negative_binomial_structure.negative_binomial_obj(
        X, counts, dispersion=d, alpha=alpha, beta=beta)

    obj_ = negative_binomial_structure.negative_binomial_obj(
        random_state.rand(*X.shape),
        counts, dispersion=d, alpha=alpha, beta=beta)
    assert(obj < obj_)


def test_negative_binomial_gradient_sparse():
    n = 10
    random_state = np.random.RandomState(42)
    X = random_state.rand(n, 3)
    dis = euclidean_distances(X)
    alpha, beta = -3, 1

    counts = beta * dis ** alpha
    counts = np.triu(counts)
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(counts)

    gradient_sparse = negative_binomial_structure.negative_binomial_gradient(
        X, counts)
    assert_array_almost_equal(np.zeros(gradient_sparse.shape), gradient_sparse)


def test_negative_binomial_gradient_sparse_dispersed():
    n = 10
    random_state = np.random.RandomState(42)
    X = random_state.rand(n, 3)
    dis = euclidean_distances(X)
    alpha, beta = -3, 1

    fdis = beta * dis**alpha
    fdis[np.isinf(fdis)] = 1
    dispersion = fdis + fdis ** 2
    p = fdis / (fdis + dispersion)

    counts = random_state.negative_binomial(dispersion, 1 - p)
    counts = np.triu(counts)
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(counts, dtype=float)
    return True
    # from minorswing import dispersion
    mean, variance = dispersion.compute_mean_variance(
        counts,
        np.array([counts.shape[0]]))
    mean, variance = mean[:-1], variance[:-1]
    d = dispersion.DispersionPolynomial()
    d.fit(mean, variance)

    gradient_sparse = negative_binomial_structure.negative_binomial_gradient(
        X, counts, dispersion=d)
    #assert_array_almost_equal(np.zeros(gradient_sparse.shape), gradient_sparse)


def test_estimate_X():
    n = 50
    random_state = np.random.RandomState(42)
    X_true = random_state.rand(n, 3)
    dis = euclidean_distances(X_true)
    alpha, beta = -3, 1

    counts = beta * dis ** alpha
    counts = np.triu(counts)
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(counts)

    X = negative_binomial_structure.estimate_X(counts, alpha, beta,
                                               random_state=random_state)
    assert_array_almost_equal(dis,
                              euclidean_distances(X), 2)


def test_estimate_X_biased_dispersion():
    n = 50
    random_state = np.random.RandomState(42)
    X_true = random_state.rand(n, 3)
    dis = euclidean_distances(X_true)
    alpha, beta = -3, 1

    fdis = beta * dis ** alpha
    fdis[np.isinf(fdis)] = 1
    dispersion = fdis + fdis ** 2
    p = fdis / (fdis + dispersion)

    counts = random_state.negative_binomial(dispersion, 1 - p)
    counts = np.triu(counts)
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(counts, dtype=np.float)

    lengths = np.array([counts.shape[0]])
    return True
    from minorswing import dispersion
    mean, variance = dispersion.compute_mean_variance(counts, lengths)
    mean, variance = mean[:-1], variance[:-1]
    d = dispersion.DispersionPolynomial()
    d.fit(mean, variance)

    X = negative_binomial_structure.estimate_X(counts, alpha, beta,
                                               dispersion=d,
                                               random_state=random_state)
    #assert_array_almost_equal(dis, euclidean_distances(X))



import os
from os import environ, makedirs
from os.path import join, expanduser, exists
from os.path import dirname
import shutil

from .. import io

# authors: Nelle Varoquaux <nelle.varoquaux@gmail.com>

# This module is greatly inspired from sklearn.datasets


def get_data_home(data_home=None):
    """Return the path of the scikit-learn data dir.

    This folder is used by some large dataset loaders to avoid
    downloading the data several times.

    By default the data dir is set to a folder named 'scikit_learn_data'
    in the user home folder.

    Alternatively, it can be set by the 'SCIKIT_LEARN_DATA' environment
    variable or programmatically by giving an explicit folder path. The
    '~' symbol is expanded to the user home folder.

    If the folder does not already exist, it is automatically created.
    """
    if data_home is None:
        data_home = environ.get('HICLIB_DATA',
                                join('~', 'hiclib_data'))
    data_home = expanduser(data_home)
    if not exists(data_home):
        makedirs(data_home)
    return data_home


def clear_data_home(data_home=None):
    """Delete all the content of the data home cache."""
    data_home = get_data_home(data_home)
    shutil.rmtree(data_home)


def load_sample_yeast():
    """
    Load and return a sample of S. cerevisiae contact count matrix from duan
    et al, Nature, 2009

    Returns
    -------
        counts, lengths:
            tuple of two elements, the first a contact count matrix, the
            second an ndarray containing the lengths of the chromosomes.
    """
    module_path = dirname(__file__)
    lengths = io.load_lengths(
        os.path.join(module_path, "data/duan2009/duan.SC.10000.raw_sub.bed"))
    counts = io.load_counts(
        os.path.join(module_path,
                     "data/duan2009/duan.SC.10000.raw_sub.matrix"),
        lengths=lengths)
    counts = counts.toarray()
    counts = counts.T + counts
    return counts, lengths



import numpy as np
from sklearn.metrics import euclidean_distances
import pyipopt

"""Utility method for the pyipopt solver, for the Poisson Model
"""

VERBOSE = False
niter = 0


def poisson_exp(X, counts, alpha, bias=None,
                beta=None, use_empty_entries=True):
    """
    Computes the log likelihood of the poisson exponential model.

    Parameters
    ----------
    X: ndarray
        3D positions

    counts: n * n ndarray
        of interaction frequencies

    alpha: float
        parameter of the expential law

    beta: float, optional, default None
        constant. If is set to None, it will be computed using the maximum log
        likelihood knowing alpha.

    use_empty_entries: boolean, optional, default False
        whether to use zeroes entries as information or not

    Returns
    -------
    ll: float
        log likelihood
    """
    if VERBOSE:
        print "Poisson power law model : computation of the log likelihood"
    m, n = X.shape
    d = euclidean_distances(X)
    if use_empty_entries:
        mask = (np.tri(m, dtype=np.bool) == False)
    else:
        mask = (np.tri(m, dtype=np.bool) == False) & (counts != 0) & (d != 0)

    if beta is None:
        beta = counts[mask].sum() / (d[mask] ** alpha).sum()
    g = beta * d[mask] ** alpha

    ll = counts[mask] * np.log(beta) + alpha * counts[mask] * np.log(d[mask])
    ll -= g
    # We are trying to maximise, so we need the opposite of the log likelihood
    return - ll.sum()


def gradient_poisson_exp(X, counts, alpha, beta, use_empty_entries=True):
    """
    Computes the gradient of the log likelihood of the gradient in alpha and
    beta

    Parameters
    ----------
    X: ndarray
        3D positions

    counts: n * n ndarray
        of interaction frequencies

    alpha: float
        parameter of the expential law

    beta: float
        constant

    use_empty_entries: boolean, optional, default: False
        whether to use the zeros entries as information or not.

    Returns
    -------
    grad_alpha, grad_beta: float, float
        The value of the gradient in alpha and the value of the gradient in
        beta
    """

    if VERBOSE:
        print "Poisson exponential model : computation of the gradient"

    m, n = X.shape

    d = euclidean_distances(X)

    if use_empty_entries:
        mask = np.tri(m, dtype=np.bool) == False
    else:
        mask = (np.tri(m, dtype=np.bool) == False) & (counts != 0)

    beta = counts[mask].sum() / (d[mask] ** alpha).sum()

    grad_alpha = - beta * (d[mask] ** alpha * np.log(d[mask])).sum() \
        + (counts[mask] * np.log(d[mask])).sum()
    return - np.array([grad_alpha])


def eval_no_f(x, user_data=None):
    """
    Evaluate the object function (no objective function).
    """
    if VERBOSE:
        print "No objective function"
    return 0.


def eval_grad_no_f(X, user_data=None):
    if VERBOSE:
        print "Gradient of no objective function"
    m, n, counts, alpha, beta, d = user_data
    X.dump('tmp.npy')
    return np.zeros(n * m)


def eval_f(x, user_data=None):
    """
    Evaluate the objective function.

    This computes the stress
    """
    if VERBOSE:
        print "Poisson exponential model : eval_f"

    m, n, counts, X, use_empty_entries = user_data
    X = X.reshape((m, n))
    tmp = poisson_exp(X, counts, x[0], use_empty_entries=use_empty_entries)
    return tmp


def eval_f_X(X, user_data=None):
    """
    Evaluate the objective function.

    This computes the stress

    Parameters
    ----------
    X: ndarray, shape m * n
        3D configuration

    user_data: optional, default=None
        m, n, counts, alpha, beta

    Returns
    -------
    loglikelihood of the model.
    """
    if VERBOSE:
        print "Poisson exponential model : computation of the eval_f"

    m, n, counts, alpha, beta, d = user_data
    X = X.reshape((m, n))
    tmp = poisson_exp(X, counts, alpha, beta)
    return tmp


def eval_grad_f(x, user_data=None):
    """
    Evaluate the gradient of the function in alpha
    """
    if VERBOSE:
        print "Poisson exponential model : eval_grad_f (evaluation in alpha)"

    m, n, counts, X, use_empty_entries = user_data
    X = X.reshape((m, n))
    tmp = gradient_poisson_exp(X, counts, x[0], None,
                               use_empty_entries=use_empty_entries)

    return tmp


def eval_grad_f_X(X, user_data=None):
    """
    Evaluate the gradient of the function in X
    """
    global niter
    niter += 1
    if not niter % 10:
        X.dump('%d.sol.npy' % niter)

    if VERBOSE:
        print "Poisson exponential model : eval_grad_f_X (evaluation in f X)"

    m, n, counts, alpha, beta, d = user_data
    X = X.reshape((m, n))
    dis = euclidean_distances(X)

    tmp = X.repeat(m, axis=0).reshape((m, m, n))
    dif = tmp - tmp.transpose(1, 0, 2)
    dis = dis.repeat(n).reshape((m, m, n))
    counts = counts.repeat(n).reshape((m, m, n))

    grad = - alpha * beta * dif / dis * (dis ** (alpha - 1)) + \
        counts * alpha * dif / (dis ** 2)
    grad[np.isnan(grad)] = 0
    return - grad.sum(1)


def eval_stress(X, user_data=None):
    """
    """
    if VERBOSE:
        print "Computing stress: eval_stress"
    m, n, distances, alpha, beta, d = user_data
    X = X.reshape((m, n))
    dis = euclidean_distances(X)
    stress = ((dis - distances) ** 2)[distances != 0].sum()
    return stress


def eval_grad_stress(X, user_data=None):
    """
    Compute the gradient of the stress
    """
    if VERBOSE:
        print 'Compute the gradient of the stress: eval_grad_stress'
    m, n, distances, alpha, beta, d = user_data
    X = X.reshape((m, n))
    tmp = X.repeat(m, axis=0).reshape((m, m, n))
    dif = tmp - tmp.transpose(1, 0, 2)
    dis = euclidean_distances(X).repeat(3, axis=1).flatten()
    distances = distances + distances.T
    distances = distances.repeat(3, axis=1).flatten()
    grad = 2 * dif.flatten() * (dis - distances) / dis
    grad[(distances == 0) | np.isnan(grad)] = 0
    return grad.reshape((m, m, n)).sum(axis=1)


def eval_g_no(x, user_data=None):
    return np.array([0, 0.])


def eval_jac_g_no(x, flag, user_data=None):
    if flag:
        return 0, 0
    return np.array([0., 0.])


def eval_g(x, user_data=None):
    """
    Computes the constraints
    """
    if VERBOSE:
        print "Poisson exponential model : eval_g"

    m, n, wish_dist, alpha, beta, d = user_data

    x = x.reshape((m, n))
    dis = euclidean_distances(x)
    dis = dis ** 2
    mask = (np.tri(m, dtype=np.bool) == False)
    g = np.concatenate([dis[mask].flatten(), ((x - d) ** 2).sum(axis=1)])
    return g


def eval_jac_g(x, flag, user_data=None):
    """
    Computes the jacobian for the constraints mentionned in duan-et-al
    """
    if VERBOSE:
        print "Poisson exponential model : eval_jac_g"

    m, n, wish_dist, alpha, beta, d = user_data

    if flag:
        ncon = m * (m - 1) / 2
        row = np.arange(ncon).repeat(2 * n)

        tmp = np.arange(n).repeat(m - 1)
        tmp = tmp.reshape((n, m - 1))
        tmp = tmp.T
        tmp1 = np.arange(n, m * n)
        tmp1 = tmp1.reshape((m - 1, n))
        tmp = np.concatenate((tmp, tmp1), axis=1)
        tmp1 = tmp.copy()
        for it in range(m):
            tmp += n
            tmp = tmp[:-1]
            tmp1 = np.concatenate((tmp1, tmp))

        # The second part of the jacobian is the restrictions on the distances
        # to the origin and/or the distances to the SPB/nucleolus center
        row_2 = np.arange(m).repeat(n)
        col = np.arange(n * m)

        col = np.concatenate([tmp1.flatten(), col])
        row = np.concatenate([row, row_2])
        return row.flatten(), col.flatten()
    else:
        x = x.reshape((m, n))
        tmp = x.repeat(m, axis=0).reshape((m, m, n))
        dif = tmp - tmp.transpose(1, 0, 2)
        mask = (np.tri(m, dtype=np.bool) == False)
        dif = dif[mask]
        jac = 2 * np.concatenate((dif, - dif), axis=1).flatten()

        # The second part of the jacobian is the restrictions on the distances
        # to the origin and/or the distances to the SPB/nucleolus center
        jac2 = 2 * (x - d).flatten()
        return np.concatenate([jac, jac2]).flatten()


def eval_h(x, lagrange, obj_factor, flag, user_data=None):
    """
    """
    return False


def estimate_alpha_beta(counts, X, ini=None, verbose=0,
                        use_empty_entries=True):
    """
    Estimate the parameters of g

    Parameters
    ----------
    counts: ndarray

    use_empty_entries: boolean, optional, default: True
        whether to use zeroes entries as information or not

    """
    m, n = X.shape
    nvar = 1
    ncon = 0
    nnzj = 0
    nnzh = 0

    x_L = np.array([- 10000.])
    x_U = np.array([10000000.])

    nlp = pyipopt.create(nvar, x_L, x_U, ncon, x_L,
                         x_U, nnzj, nnzh, eval_f,
                         eval_grad_f, eval_g, eval_jac_g)

    nlp.int_option('max_iter', 100)
    if ini is None:
        if verbose:
            print "Initial values not provided"
        ini = np.random.randint(1, 100, size=(1, )) + \
            np.random.random(size=(1, ))
    results = nlp.solve(ini, (m, n, counts, X, use_empty_entries))
    try:
        x, _, _, _, _ = results
    except ValueError:
        x, _, _, _, _, _ = results

    # Evaluate counts with new estimated model.
    d = euclidean_distances(X)
    mask = (np.tri(m, dtype=np.bool) == False) & (counts != 0) & (d != 0)
    beta = counts[mask].sum() / (d[mask] ** x[0]).sum()
    return x[0], beta


# Let's now work on reconstructing the 3D structure
def estimate_3D_structure(nlp, counts, alpha, beta, d, X=None, m=None,
                          n=None):
    """
    Maximise the loglikelihood in X

    """

    if X is None:
        X = np.random.random(size=(m * n))

    X, _, _, _, _ = nlp.solve(X.flatten(), (m, n, counts, alpha, beta, d))
    X = X.reshape((m, n))
    return X



# License: BSD Style.
import os
from os.path import join

import numpy


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    libraries = []
    if os.name == 'posix':
        libraries.append('m')

    config = Configuration('io', parent_package, top_path)

    config.add_extension(
        'fastio_',
        libraries=libraries,
        sources=['fastio_.c'],
        include_dirs=[join('..', 'src', 'cblas'),
                      numpy.get_include()])
    config.add_extension(
        'read',
        libraries=libraries,
        sources=['read.c'],
        include_dirs=[join('..', 'src', 'cblas'),
                      numpy.get_include()])

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



import numpy as np
from scipy import sparse
from .fastio_ import loadtxt


def load_counts(filename, lengths=None):
    """
    Fast loading of a raw interaction counts file

    Parameters
    ----------
    filename : str
        path to the file to load. The file should be of the following format:
        i, j, counts

    lengths : ndarray
        lengths of each chromosomes

    Returns
    --------
    X : the interaction counts file
    """
    n = None
    if lengths is not None:
        n = lengths.sum()
        shape = (n, n)
    else:
        shape = None
    X = loadtxt(filename.encode())
    X = X[X[:, 2] != 0]
    if shape is not None:
        if X[:, :2].max() == shape[0]:
            X[:, :2] -= 1
    counts = sparse.coo_matrix((X[:, 2], (X[:, 0], X[:, 1])),
                               shape=shape, dtype=np.float64)
    return counts


def load_lengths(filename):
    """
    Fast loading of the bed files

    Parameters
    ----------
    filename : str,
        path to the file to load. The file should be a bed file

    Returns
    -------
    lengths : the lengths of each chromosomes
    """
    data = np.loadtxt(filename, dtype="str")
    lengths = [(data[:, 0] == i).sum() for i in np.unique(data[:, 0])]
    return np.array(lengths)


def write_counts(filename, counts):
    """
    Write counts

    Parameters
    ----------

    filename : str

    counts: array-like
    """
    raise NotImplementedError



import os
import tempfile
from iced.datasets import clear_data_home, get_data_home
from iced.datasets import load_sample_yeast
from nose.tools import assert_true, assert_false, assert_equal

DATA_HOME = tempfile.mkdtemp(prefix="hiclib_data_home_test_")
LOAD_FILES_ROOT = tempfile.mkdtemp(prefix="hiclib_load_files_test_")
TEST_CATEGORY_DIR1 = ""
TEST_CATEGORY_DIR2 = ""


def test_data_home():
    # get_data_home will point to a pre-existing folder
    data_home = get_data_home(data_home=DATA_HOME)
    assert_equal(data_home, DATA_HOME)
    assert_true(os.path.exists(data_home))

    # clear_data_home will delete both the content and the folder it-self
    clear_data_home(data_home=data_home)
    assert_false(os.path.exists(data_home))

    # if the folder is missing it will be created again
    data_home = get_data_home(data_home=DATA_HOME)
    assert_true(os.path.exists(data_home))

def test_data_sub_yeast():
    counts, lengths = load_sample_yeast()



import numpy as np
from sklearn.metrics import euclidean_distances
from numpy.testing import assert_array_almost_equal
from scipy import sparse
from pastis.optimization import poisson_structure


def test_estimate_X_with_diag():
    n = 25
    random_state = np.random.RandomState(42)
    X_true = random_state.rand(n, 3)
    dis = euclidean_distances(X_true)
    alpha, beta = -3., 1.

    counts = beta * dis ** alpha
    counts[np.isnan(counts) | np.isinf(counts)] = 0
    counts[np.arange(len(counts)), np.arange(len(counts))] = 5
    counts = np.triu(counts)
    counts = sparse.coo_matrix(counts)

    X = poisson_structure.estimate_X(counts, random_state=random_state)


def test_estimate_X_ini():
    n = 25
    random_state = np.random.RandomState(42)
    X_true = random_state.rand(n, 3)
    dis = euclidean_distances(X_true)
    alpha, beta = -3., 1.
    ini = list(np.random.randn(n * 3))

    counts = beta * dis ** alpha
    counts[np.isnan(counts) | np.isinf(counts)] = 0
    counts[np.arange(len(counts)), np.arange(len(counts))] = 5
    counts = np.triu(counts)
    counts = sparse.coo_matrix(counts)

    X = poisson_structure.estimate_X(counts, random_state=random_state,
                                     ini=ini)



import numpy as np
from scipy import sparse
from ._filter_ import _filter_csr
from . import utils


def filter_low_counts(X, lengths=None, percentage=0.02, copy=True,
                      sparsity=True, remove_all_zeros_loci=False,
                      verbose=False):
    """
    Filter rows and columns with low counts

    Parameters
    ----------
    X : ndarray (n, n)
        Count matrix (hollow, symetric)

    lengths : ndarray (m, ), optional, default: None
        Lengths of the chromosomes

    percentage : float, optional, default: 0.02
        percentage of rows and columns to discard

    remove_all_zeros_loci : bool, optional, default: False
        if set to True, the filtering will remove first all the non
        interacting loci, and then apply the filtering strategy chosen.

    copy : boolean, optional, default: True
        If set to true, copies the count matrix

    sparsity : boolean, optional, default: True
        Whether to use the sparsity of the region or the total number of
        counts of the region to filer.

    Return
    ------
    X : ndarray (n, n)
        The filtered array
    """
    if sparse.issparse(X):
        if not sparse.isspmatrix_csr(X):
            X = X.tocsr()
        X.sort_indices()
    else:
        X[np.isnan(X)] = 0

    if sparsity:
        if lengths is not None:
            weights = []
            [weights.append(i) for i in lengths for j in range(i)]
            weights = np.array(weights)
            mask = utils.get_inter_mask(lengths)
        else:
            weights = np.ones(X.shape[0])
            mask = np.zeros(X.shape, dtype=np.bool)

        return _filter_low_sparse(X, weights, mask, percentage=percentage,
                                  remove_all_zeros_loci=remove_all_zeros_loci,
                                  verbose=verbose)
    else:
        return _filter_low_sum(X, percentage=percentage,
                               remove_all_zeros_loci=remove_all_zeros_loci,
                               verbose=verbose)


def filter_high_counts(X, lengths=None, percentage=0.02, copy=True):
    """
    Filter rows and columns with high counts

    Parameters
    ----------
    X : ndarray (n, n)
        Count matrix (hollow, symetric)

    lengths : ndarray (m, ), optional, default: None
        Lengths of the chromosomes

    percentage : float, optional, default: 0.02
        percentage of rows and columns to discard

    copy : boolean, optional, default: True
        If set to true, copies the count matrix

    Return
    ------
    X : ndarray (n, n)
        The filtered array

    Notes
    -----
    New in 0.6
    """
    if sparse.issparse(X):
        if not sparse.isspmatrix_csr(X):
            X = X.tocsr()
        X.sort_indices()
    else:
        X[np.isnan(X)] = 0

    return _filter_high_sum(X, percentage=percentage)


def _filter_low_sparse(X, weights, mask, percentage=0.02,
                       remove_all_zeros_loci=False, verbose=False):
    # This is NOT going to work on sparse data. For now, raise a Not
    # implemented error

    if sparse.issparse(X):
        raise NotImplemented
    if remove_all_zeros_loci:
        raise NotImplemented

    counts = X.copy()
    counts[mask] = 1
    X_sum = (counts == 0).sum(axis=0).astype(float) / weights
    X_sum.sort()
    X_sum = np.array(X_sum).flatten()
    x = X_sum[int(X.shape[0] * (1. - percentage))]
    X_sum = (counts == 0).sum(axis=0).astype(float) / weights

    if sparse.issparse(X):
        _filter_csr(X, (X_sum < x))
    else:
        X[X_sum > x, :] = np.nan
        X[:, X_sum > x] = np.nan

    return X


def _filter_high_sum(X, percentage=0.02, verbose=False):
    X_sum = (np.array(X.sum(axis=0)).flatten() +
             np.array(X.sum(axis=1)).flatten())
    X_sum.sort()
    m = X.shape[0]
    x = X_sum[int(m * (1-percentage))]

    if verbose:
        print("Filter %s bins ..." % sum(X_sum > x))

    X_sum = (np.array(X.sum(axis=0)).flatten() +
             np.array(X.sum(axis=1)).flatten())

    if sparse.issparse(X):
        _filter_csr(X, (X_sum > x))
    else:
        X[X_sum > x, :] = np.nan
        X[:, X_sum > x] = np.nan

    return X


def _filter_low_sum(X, percentage=0.02, remove_all_zeros_loci=False,
                    verbose=False):
    X_sum = (np.array(X.sum(axis=0)).flatten() +
             np.array(X.sum(axis=1)).flatten())
    X_sum.sort()
    m = X.shape[0]

    if not remove_all_zeros_loci:
        x = X_sum[int(m * percentage)]
    else:
        num_noninteracting_loci = sum(X_sum == 0)
        x = X_sum[
            int(len(X_sum[X_sum > 0]) * percentage) + num_noninteracting_loci]

    X_sum = (np.array(X.sum(axis=0)).flatten() +
             np.array(X.sum(axis=1)).flatten())
    if verbose:
        print("Filter %s out of %s bins ..." % (sum(X_sum < x), m))

    if sparse.issparse(X):
        _filter_csr(X, (X_sum < x))
    else:
        X[X_sum < x, :] = np.nan
        X[:, X_sum < x] = np.nan

    return X



import numpy as np
from scipy.ndimage import gaussian_filter


def smooth(counts, lengths,  sigma=1):
    begin_i, end_i = 0, 0
    for i, l_i in enumerate(lengths):
        end_i = l_i
        begin_j, end_j = 0, 0
        for j, l_j in enumerate(lengths):
            end_j = l_j
            sc = counts[begin_i:end_i, begin_j:end_j]
            counts[begin_i:end_i, begin_j:end_j] = gaussian_filter(sc, sigma)
            begin_j = l_j
        begin_i = end_i
    return counts


def average(counts, lengths):
    begin_i, end_i = 0, 0
    for i, l_i in enumerate(lengths):
        end_i = l_i
        begin_j, end_j = 0, 0
        for j, l_j in enumerate(lengths):
            end_j = l_j
            sc = counts[begin_i:end_i, begin_j:end_j]
            counts[begin_i:end_i, begin_j:end_j] = np.nanmean(sc)
            begin_j = l_j
        begin_i = end_i
    return counts



import numpy as np
from scipy import sparse
from nose.tools import assert_raises
from iced.utils import validation


def test_is_symetric_or_tri():
    n = 100
    m = 50
    random_state = np.random.RandomState(seed=42)
    X = random_state.randn(n, m)
    assert_raises(ValueError, validation.is_symetric_or_tri, X)
    X = random_state.randn(n, n)
    assert_raises(ValueError, validation.is_symetric_or_tri, X)
    X = X + X.T
    validation.is_symetric_or_tri(X)
    X = np.triu(X)
    validation.is_symetric_or_tri(X)


def test_is_symetric_or_tri_sparse():
    n = 100
    m = 50
    random_state = np.random.RandomState(seed=42)
    X = sparse.csr_matrix(random_state.randn(n, m))
    assert_raises(ValueError, validation.is_symetric_or_tri, X)
    X = sparse.csr_matrix(random_state.randn(n, n))
    assert_raises(ValueError, validation.is_symetric_or_tri, X)
    X = random_state.randn(n, n)
    X = X + X.T
    X = sparse.csr_matrix(X)
    validation.is_symetric_or_tri(X)
    X[np.tri(n, dtype=bool)] = 0
    validation.is_symetric_or_tri(X)


def test_is_tri():
    n = 100
    random_state = np.random.RandomState(seed=42)
    X = random_state.randn(n, n)
    assert validation.is_tri(np.triu(X))
    assert validation.is_tri(np.tril(X))



import numpy as np
from scipy import sparse
from scipy import special
from scipy import optimize
from sklearn.utils import check_random_state
from .utils import ConstantDispersion

_niter = 0


def negative_binomial_obj(X, counts, alpha=-3., beta=1., bias=None,
                          dispersion=None,
                          use_zero_counts=False, cst=0):
    if bias is None:
        bias = np.ones((counts.shape[0], 1))

    if dispersion is None:
        dispersion = ConstantDispersion()

    if sparse.issparse(counts):
        return _negative_binomial_obj_sparse(
            X, counts, beta=beta, bias=bias,
            alpha=alpha, dispersion=dispersion,
            use_zero_counts=use_zero_counts, cst=cst)
    else:
        return _negative_binomial_obj_dense(
            X, counts, beta=beta, bias=bias,
            alpha=alpha, d=dispersion, use_zero_counts=use_zero_counts)


def _negative_binomial_obj_dense(X, counts, alpha=-3, beta=1, d=7, bias=None,
                                 use_zero_counts=False):
    raise NotImplementedError


def _negative_binomial_obj_sparse(X, counts, alpha=-3, beta=1., bias=None,
                                  dispersion=None,
                                  use_zero_counts=False, cst=0):
    if use_zero_counts:
        raise NotImplementedError

    bias = bias.flatten()

    # XXX This will return nan if the counts data is not hollow !!!
    # We should check for this

    dis = np.sqrt(((X[counts.row] - X[counts.col])**2).sum(axis=1))
    fdis = bias[counts.row] * bias[counts.col] * beta * dis ** alpha

    d = dispersion.predict(fdis)

    obj = - special.gammaln(counts.data + d).sum()
    obj += special.gammaln(d).sum()
    obj -= (counts.data * np.log(fdis)).sum()
    obj -= (d * np.log(d)).sum()
    obj += ((counts.data + d) * np.log(d + fdis)).sum()
    return obj.sum()


def negative_binomial_gradient(X, counts, alpha=-3, beta=1, bias=None,
                               dispersion=None,
                               use_zero_counts=False):
    if bias is None:
        bias = np.ones((counts.shape[0], 1))

    if dispersion is None:
        dispersion = ConstantDispersion()

    if sparse.issparse(counts):
        return _negative_binomial_gradient_sparse(
            X, counts, beta=beta, bias=bias,
            alpha=alpha, dispersion=dispersion,
            use_zero_counts=use_zero_counts)
    else:
        return _negative_binomial_gradient_dense(
            X, counts, beta=beta, bias=bias,
            alpha=alpha, d=dispersion,
            use_zero_counts=use_zero_counts)


def _negative_binomial_gradient_dense(X, counts, alpha=-3, beta=1, d=7,
                                      bias=None,
                                      use_zero_counts=False):
    raise NotImplementedError


def _negative_binomial_gradient_sparse(X, counts, alpha=-3, beta=1.,
                                       dispersion=None,
                                       bias=None,
                                       use_zero_counts=False):
    if use_zero_counts:
        raise NotImplementedError
    bias = bias.flatten()

    dis = np.sqrt(((X[counts.row] - X[counts.col])**2).sum(axis=1))
    fdis = bias[counts.row] * bias[counts.col] * beta * dis ** alpha

    diff = X[counts.row] - X[counts.col]

    d = dispersion.predict(fdis)

    d_prime = (dispersion.derivate(fdis) * alpha * beta * bias[counts.row] *
               bias[counts.col] * dis ** (alpha - 2))[:, np.newaxis] * diff

    grad = -(special.digamma(counts.data + d)[:, np.newaxis] * d_prime)
    grad += special.digamma(d)[:, np.newaxis] * d_prime
    grad -= (counts.data * alpha / dis ** 2)[:, np.newaxis] * diff
    grad -= (np.log(d) + 1)[:, np.newaxis] * d_prime
    grad += np.log(d + fdis)[:, np.newaxis] * d_prime
    grad += ((counts.data + d) / (d + fdis))[:, np.newaxis] * (
        (fdis * alpha / dis**2)[:, np.newaxis] * diff + d_prime)

    grad_ = np.zeros(X.shape)

    for i in range(X.shape[0]):
        grad_[i] += grad[counts.row == i].sum(axis=0)
        grad_[i] -= grad[counts.col == i].sum(axis=0)

    return grad_


def eval_f(x, user_data=None):
    n, counts, alpha, beta, dispersion, bias, use_zero_counts, cst = user_data
    x = x.reshape((n, 3))
    obj = negative_binomial_obj(x, counts, alpha=alpha, beta=beta, bias=bias,
                                dispersion=dispersion, cst=cst)
    return obj


def eval_grad_f(x, user_data=None):
    n, counts, alpha, beta, dispersion, bias, use_zero_counts, cst = user_data
    x = x.reshape((n, 3))
    grad = negative_binomial_gradient(x, counts, alpha=alpha,
                                      beta=beta, bias=bias,
                                      dispersion=dispersion)
    x = x.flatten()
    return grad.flatten()


def estimate_X(counts, alpha, beta,
               ini=None, dispersion=None, bias=None,
               verbose=0,
               use_zero_entries=False,
               random_state=None, maxiter=10000):
    """
    Estimate the parameters of g

    Parameters
    ----------
    counts: ndarray

    """
    n = counts.shape[0]

    random_state = check_random_state(random_state)
    if ini is None:
        ini = 1 - 2 * random_state.rand(n * 3)

    data = (n, counts, alpha, beta, dispersion, bias,
            use_zero_entries, 0)

    results = optimize.fmin_l_bfgs_b(
        eval_f,
        ini.flatten(),
        eval_grad_f,
        (data, ),
        iprint=0,
        maxiter=maxiter,
        )
    results = results[0].reshape(-1, 3)
    return results



import numpy as np


def write(X, filename, lengths=None,
          resolution=50000,
          copy=True):
    """
    Write the similarity matrix to a file

    Parameters
    ----------
    X: ndarray,
    """
    if copy:
        X = X.copy()
    X = X.astype(float)

    output_file = open(filename, 'w')

    tr = X.sum(axis=0) == 0
    if tr.sum() != 0:
        X[tr, :] = np.nan
        X[:, tr] = np.nan

    for i in range(len(X)):
        chr1, loc1 = _get_index(i, lengths)
        if loc1 > lengths[chr1 - 1]:
            raise ValueError("Problem !")
        loc1 *= resolution
        loc1 += 1
        for j in range(len(X)):
            if i > j:
                continue
            chr2, loc2 = _get_index(j, lengths)
            if loc2 > lengths[chr2 - 1]:
                raise ValueError("Problem !")
            loc2 *= resolution
            loc2 += 1

            if chr2 == chr1:
                dis = loc2 - loc1
            else:
                dis = 0

            if not X[i, j] or np.isnan(X[i, j]):
                continue

            output_file.write('%d\t%d\t%d\t%d\t%d\t%f\t0\t0\n' %
                              (chr1, loc1, chr2, loc2, dis, X[i, j]))

    output_file.close()


def _get_index(i, lengths):
    """
    From an index, return the chromosome number and the loci number

    Parameters
    ----------
    i : integer
        index in the full matrix

    lengths : ndarray (n, )

    Returns
    -------
    tuple : (c, l)
        chromosome number, locus
    """
    length_cum = lengths.cumsum()
    c = (length_cum > i).argmax() + 1

    if c > 1:
        length = i - length_cum[c - 2]
    else:
        length = i
    if len(lengths) > 1 and length > lengths[c - 1]:
        raise ValueError("there's a problem")

    if c > len(lengths):
        raise ValueError("There's a problem")
    return c, length


###############################################################################
# 160907 GBonora
# Python implementation of 'print_pdb_atom' function in coords.cpp

def fprintf(fp, fmt, *args):
    """
    Simply an alias for fprintf to allow Python implementation to remain similar to original C version.

    Parameters
    ----------
    fp : output file

    fmt : C-style format string

    args: arguments for format string

    """
    fp.write(fmt % args)


def print_pdb_atom(outfile,
                   # chrom_index,
                   # copy_index,
                   atom_index,
                   is_node,
                   atom_name,
                   scale_factor,
                   my_coords):
    """
    Prints one line to PDB file.

    Parameters
    ----------
    outfile : output file

    atom_index : integer
        'atom' ID

    is_node : boolean
        Is this a node or an edge atom?

    atom_name : string
        eg "N", "O", or "C"

    scale_factor : integer
        100 for a sphere with radius 1, 1000 for a sphere with radius 10

    my_coords :
        3D co-ordinates to ouput


    """
    # prev_chrom_index = -1
    # atom_index
    # if chrom_index != prev_chrom_index:
    #     atom_index = 1
    #     prev_chrom_index = chrom_index

    # http://www.biochem.ucl.ac.uk/~roman/procheck/manual/manappb.html
    fprintf(outfile, "ATOM  ")  # 1- 6: Record ID
    fprintf(outfile, "%5d", atom_index)  # 7-11: Atom serial number
    fprintf(outfile, " ")  # 12: Blank
    # if (is_node) {                        # 13-16: Atom name
    #   fprintf(outfile, "N   ")
    # } else {
    #   fprintf(outfile, "O   ")
    # }
    fprintf(outfile, "%s   ", atom_name)
    fprintf(outfile, " ")  # 17-17: Alternative location code
    if is_node:  # 18-20: 3-letter amino acid code
        fprintf(outfile, "NOD")
    else:
        fprintf(outfile, "EDG")
    fprintf(outfile, " ")  # 21: Blank
    fprintf(outfile, "%c",  # 22: Chain identifier code
            # get_chrom_id(chrom_index, copy_index))
            'A')
    fprintf(outfile, "    ")  # 23-26: Residue sequence number
    fprintf(outfile, " ")  # 27: Insertion code
    fprintf(outfile, "   ")  # 28-30: Blank
    fprintf(outfile, "%8.3f%8.3f%8.3f",  # 31-54: Atom coordinates
            # (my_coords->x + 1.0) * SCALE_FACTOR,
            # (my_coords->y + 1.0) * SCALE_FACTOR,
            # (my_coords->z + 1.0) * SCALE_FACTOR)
            (my_coords[0] + 1.0) * scale_factor,
            (my_coords[1] + 1.0) * scale_factor,
            (my_coords[2] + 1.0) * scale_factor)
    fprintf(outfile, "%6.2f", 1.0)  # 55-60: Occupancy value
    if is_node:
        fprintf(outfile, "%6.2f", 50.0)  # 61-66: B-value (thermal factor)
    else:
        fprintf(outfile, "%6.2f", 75.0)  # 61-66: B-value (thermal factor)
    fprintf(outfile, " ")  # 67: Blank
    fprintf(outfile, "   ")  # 68-70: Blank
    fprintf(outfile, "\n")


def writePDB(Xpdb, pdbfilename):
    """
    Write 3D ouput from pastis to PDB file.

    Parameters
    ----------
    Xpdb : nparray
        the 3D co-ordinates

    pdbfilename : File to write PDB file to.

    """
    # print('PDB file creation!')
    atom_name = 'O'
    scale_factor = 100  # 100 for a sphere with radius 1, 1000 for a sphere with radius 10
    # pdboutfile = open(pdbfilename,'w')
    with open(pdbfilename, "w") as pdboutfile:
        for coordIdx in range(0, np.shape(Xpdb)[0]):
            # print coordIdx
            if coordIdx == 0 or coordIdx == np.shape(Xpdb)[0]:
                is_node = True
            else:
                is_node = False
            my_coords = Xpdb[coordIdx, :]
            print_pdb_atom(pdboutfile,
                           # chrom_index,
                           # copy_index,
                           coordIdx,
                           is_node,  # Is this a node or an edge atom
                           atom_name,  # eg "N", "O", or "C",
                           scale_factor,
                           my_coords)
    # pdboutfile.close()



import numpy as np
from scipy import sparse


def is_symetric_or_tri(X, eps=1e-7):
    m, n = X.shape
    if m != n:
        raise ValueError("The matrix should be of shape (n, n)")

    if is_tri(X):
        return True
    if np.abs(X - X.T).sum() > eps:
        raise ValueError("The matrix should be symmetric")


def is_tri(X):
    diag = X.diagonal().sum()
    if sparse.issparse(X):
        if not (sparse.tril(X).sum() - diag) or \
           not (sparse.triu(X).sum() - diag):
            return True
    elif not np.triu(X, 1).sum() or not np.tril(X, -1).sum():
        return True
    else:
        return False



from pastis import config
import os


def test_parse():
    config.parse()
    options = config.parse(
        os.path.abspath(
            os.path.join(
                os.path.dirname(__file__),
                "config.ini")))


def test_get_default_options():
    options = config.get_default_options()



import numpy as np
from sklearn.metrics import euclidean_distances
from scipy import sparse


class ConstantDispersion(object):
    def __init__(self, coef=7):
        self.coef = coef

    def fit(self, X, y):
        return self

    def predict(self, X):
        return self.coef * np.ones(X.shape)

    def derivate(self, X):
        return np.zeros(X.shape)


def compute_wish_distances(counts, alpha=-3., beta=1., bias=None):
    """
    Computes wish distances from a counts matrix

    Parameters
    ----------
    counts : ndarray
        Interaction counts matrix

    alpha : float, optional, default: -3
        Coefficient of the power law

    beta : float, optional, default: 1
        Scaling factor

    Returns
    -------
    wish_distances
    """
    if beta == 0:
        raise ValueError("beta cannot be equal to 0.")
    counts = counts.copy()
    if sparse.issparse(counts):
        if not sparse.isspmatrix_coo(counts):
            counts = counts.tocoo()
        if bias is not None:
            bias = bias.flatten()
            counts.data /= bias[counts.row] * bias[counts.col]
        wish_distances = counts / beta
        wish_distances.data[wish_distances.data != 0] **= 1. / alpha
        return wish_distances
    else:
        wish_distances = counts.copy() / beta
        wish_distances[wish_distances != 0] **= 1. / alpha

        return wish_distances


def eval_no_f(x, user_data=None):
    """
    Evaluate the object function (no objective function).
    """
    return 0.


def eval_grad_no_f(X, user_data=None):
    m, n, counts, alpha, beta, d = user_data
    return np.zeros(n * m)


def eval_g_no(x, user_data=None):
    return np.array([0, 0.])


def eval_jac_g_no(x, flag, user_data=None):
    if flag:
        return 0, 0
    return np.array([0., 0.])


def eval_g(x, user_data=None):
    """
    Computes the constraints
    """

    m, n, wish_dist, alpha, beta, d = user_data

    x = x.reshape((m, n))
    dis = euclidean_distances(x)
    dis = dis ** 2
    mask = np.invert(np.tri(m, dtype=np.bool))
    g = np.concatenate([dis[mask].flatten(), ((x - d) ** 2).sum(axis=1)])
    return g


def eval_jac_g(x, flag, user_data=None):
    """
    Computes the jacobian for the constraints mentionned in duan-et-al
    """

    m, n, wish_dist, alpha, beta, d = user_data

    if flag:
        ncon = m * (m - 1) / 2
        row = np.arange(ncon).repeat(2 * n)

        tmp = np.arange(n).repeat(m - 1)
        tmp = tmp.reshape((n, m - 1))
        tmp = tmp.T
        tmp1 = np.arange(n, m * n)
        tmp1 = tmp1.reshape((m - 1, n))
        tmp = np.concatenate((tmp, tmp1), axis=1)
        tmp1 = tmp.copy()
        for it in range(m):
            tmp += n
            tmp = tmp[:-1]
            tmp1 = np.concatenate((tmp1, tmp))

        # The second part of the jacobian is the restrictions on the distances
        # to the origin and/or the distances to the SPB/nucleolus center
        row_2 = np.arange(m).repeat(n)
        col = np.arange(n * m)

        col = np.concatenate([tmp1.flatten(), col])
        row = np.concatenate([row, row_2])
        return row.flatten(), col.flatten()
    else:
        x = x.reshape((m, n))
        tmp = x.repeat(m, axis=0).reshape((m, m, n))
        dif = tmp - tmp.transpose(1, 0, 2)
        mask = np.invert(np.tri(m, dtype=np.bool))
        dif = dif[mask]
        jac = 2 * np.concatenate((dif, - dif), axis=1).flatten()

        # The second part of the jacobian is the restrictions on the distances
        # to the origin and/or the distances to the SPB/nucleolus center
        jac2 = 2 * (x - d).flatten()
        return np.concatenate([jac, jac2]).flatten()


def eval_h(x, lagrange, obj_factor, flag, user_data=None):
    """
    """
    return False



import os
from numpy.distutils.core import setup


DISTNAME = 'iced'
DESCRIPTION = 'ICE normalization'
MAINTAINER = 'Nelle Varoquaux'
MAINTAINER_EMAIL = 'nelle.varoquaux@gmail.com'
VERSION = '0.4.2'


def configuration(parent_package='', top_path=None):
    if os.path.exists('MANIFEST'):
        os.remove('MANIFEST')

    from numpy.distutils.misc_util import Configuration
    config = Configuration("externals", parent_package, top_path)

    config.add_subpackage('iced')

    return config


if __name__ == "__main__":
    setup(configuration=configuration,
          name=DISTNAME,
          maintainer=MAINTAINER,
          include_package_data=True,
          maintainer_email=MAINTAINER_EMAIL,
          description=DESCRIPTION,
          version=VERSION,
          zip_safe=False,  # the package can run out of an .egg file
          scripts=['iced/scripts/ice'],
          classifiers=[
              'Intended Audience :: Science/Research',
              'Intended Audience :: Developers',
              'License :: OSI Approved',
              'Programming Language :: C',
              'Programming Language :: Python',
              'Topic :: Software Development',
              'Topic :: Scientific/Engineering',
              'Operating System :: Microsoft :: Windows',
              'Operating System :: POSIX',
              'Operating System :: Unix',
              'Operating System :: MacOS'])



import numpy as np
from scipy import sparse
from sklearn.metrics import euclidean_distances
from numpy.testing import assert_array_almost_equal
from pastis.optimization import poisson_model


def test_poisson_exp():
    random_state = np.random.RandomState(seed=42)
    n = 50
    X = random_state.rand(n, 3)
    counts = euclidean_distances(X)**(-3)
    counts[np.isinf(counts) | np.isnan(counts)] = 0
    eps = poisson_model.poisson_exp(X, counts, -2)
    assert eps < 1e-6


def test_poisson_exp_biased():
    random_state = np.random.RandomState(seed=42)
    n = 50
    bias = 0.1 + random_state.rand(n)
    bias = bias.reshape(n, 1)
    X = random_state.rand(n, 3)
    counts = euclidean_distances(X)**(-3)
    counts *= bias * bias.T
    counts[np.isinf(counts) | np.isnan(counts)] = 0
    eps = poisson_model.poisson_exp(X, counts, -2, bias=bias)
    assert eps < 1e-6


def test_poisson_exp_sparse():
    random_state = np.random.RandomState(seed=42)
    n = 50
    X = random_state.rand(n, 3)
    counts = euclidean_distances(X)**(-3)
    counts[np.isinf(counts) | np.isnan(counts)] = 0
    counts_dense = counts
    counts_sparse = sparse.coo_matrix(np.triu(counts))
    exp_dense = poisson_model.poisson_exp(X, counts_dense, -3,
                                          use_empty_entries=False)
    exp_sparse = poisson_model.poisson_exp(X, counts_sparse, -3,
                                           use_empty_entries=False)
    assert_array_almost_equal(exp_dense, exp_sparse)


def test_poisson_exp_sparse_biased():
    random_state = np.random.RandomState(seed=42)
    n = 50
    X = random_state.rand(n, 3)
    bias = 0.1 + random_state.rand(n)
    bias = bias.reshape(n, 1)

    counts = euclidean_distances(X)**(-3)
    counts[np.isinf(counts) | np.isnan(counts)] = 0
    counts *= bias * bias.T

    counts_dense = counts
    counts_sparse = sparse.coo_matrix(np.triu(counts))
    exp_dense = poisson_model.poisson_exp(X, counts_dense, -3, bias=bias,
                                          use_empty_entries=False)
    exp_sparse = poisson_model.poisson_exp(X, counts_sparse, -3, bias=bias,
                                           use_empty_entries=False)
    assert_array_almost_equal(exp_dense, exp_sparse)


def test_gradient_poisson_exp_sparse():
    random_state = np.random.RandomState(seed=42)
    n = 50
    X = random_state.rand(n, 3)
    counts = euclidean_distances(X)**(-3)
    counts[np.isinf(counts) | np.isnan(counts)] = 0
    counts_dense = counts
    counts_sparse = sparse.coo_matrix(np.triu(counts))
    grad_dense = poisson_model.gradient_poisson_exp(X, counts_dense, -3,
                                                    beta=1,
                                                    use_empty_entries=False)
    grad_sparse = poisson_model.gradient_poisson_exp(X, counts_sparse, -3,
                                                     beta=1,
                                                     use_empty_entries=False)
    assert_array_almost_equal(grad_dense, grad_sparse)


def test_gradient_poisson_exp_sparse_biased():
    random_state = np.random.RandomState(seed=42)
    n = 50
    X = random_state.rand(n, 3)
    bias = 0.1 + random_state.rand(n)
    bias = bias.reshape(n, 1)

    counts = euclidean_distances(X)**(-3)
    counts[np.isinf(counts) | np.isnan(counts)] = 0
    counts *= bias * bias.T

    counts_dense = counts
    counts_sparse = sparse.coo_matrix(np.triu(counts))
    grad_dense = poisson_model.gradient_poisson_exp(X, counts_dense, -3,
                                                    bias=bias,
                                                    beta=1,
                                                    use_empty_entries=False)
    grad_sparse = poisson_model.gradient_poisson_exp(X, counts_sparse, -3,
                                                     bias=bias,
                                                     beta=1,
                                                     use_empty_entries=False)
    assert_array_almost_equal(np.array([0]), grad_dense)
    assert_array_almost_equal(grad_dense, grad_sparse)


def test_estimate_alpha_beta():
    n = 100
    random_state = np.random.RandomState(seed=42)
    X = random_state.rand(n, 3)
    dis = euclidean_distances(X)
    beta_true, alpha_true = 2., -3.
    counts = beta_true * dis ** alpha_true
    counts[np.isnan(counts) | np.isinf(counts)] = 0
    counts = np.triu(counts)

    alpha, beta = poisson_model.estimate_alpha_beta(
        sparse.coo_matrix(counts), X)
    assert_array_almost_equal(alpha_true, alpha, 5)
    assert_array_almost_equal(beta_true, beta, 5)


def test_estimate_alpha_beta_biased():
    n = 100
    random_state = np.random.RandomState(seed=42)
    X = random_state.rand(n, 3)
    dis = euclidean_distances(X)
    bias = 0.1 + random_state.rand(n)
    bias = bias.reshape(n, 1)
    beta_true, alpha_true = 2., -3.5
    counts = beta_true * dis ** alpha_true
    counts *= bias * bias.T
    counts[np.isnan(counts) | np.isinf(counts)] = 0
    counts = np.triu(counts)

    alpha, beta = poisson_model.estimate_alpha_beta(
        sparse.coo_matrix(counts), X, bias=bias)
    assert_array_almost_equal(alpha_true, alpha, 5)
    assert_array_almost_equal(beta_true, beta, 4)




from scipy import sparse


def compute_wish_distances(counts, alpha=-3., beta=1., bias=None):
    """
    Computes wish distances from a counts matrix

    Parameters
    ----------
    counts : ndarray
        Interaction counts matrix

    alpha : float, optional, default: -3
        Coefficient of the power law

    beta : float, optional, default: 1
        Scaling factor

    Returns
    -------
    wish_distances
    """
    if beta == 0:
        raise ValueError("beta cannot be equal to 0.")
    counts = counts.copy()
    if sparse.issparse(counts):
        if not sparse.isspmatrix_coo(counts):
            counts = counts.tocoo()
        if bias is not None:
            bias = bias.flatten()
            counts.data /= bias[counts.row] * bias[counts.col]
        wish_distances = counts / beta
        wish_distances.data[wish_distances.data != 0] **= 1. / alpha
        return wish_distances
    else:
        wish_distances = counts.copy() / beta
        wish_distances[wish_distances != 0] **= 1. / alpha

        return wish_distances



import os
import sys
import shutil
from distutils.command.clean import clean as Clean


DISTNAME = 'pastis'
DESCRIPTION = 'A set of algorithms for the 3D inference of the genome'
MAINTAINER = 'Nelle Varoquaux'
MAINTAINER_EMAIL = 'nelle.varoquaux@ensmp.fr'
VERSION = '0.3.3alpha'
LICENSE = "New BSD"
URL = 'http://cbio.ensmp.fr/pastis'
DOWNLOAD_URL = 'https://github.com/hiclib/pastis/releases'

SCIPY_MIN_VERSION = '0.13.3'
NUMPY_MIN_VERSION = '1.9.0'


# Optional setuptools features
# We need to import setuptools early, if we want setuptools features,
# as it monkey-patches the 'setup' function
# For some commands, use setuptools
SETUPTOOLS_COMMANDS = set([
    'develop', 'release', 'bdist_egg', 'bdist_rpm',
    'bdist_wininst', 'install_egg_info', 'build_sphinx',
    'egg_info', 'easy_install', 'upload', 'bdist_wheel',
    '--single-version-externally-managed',
])

if SETUPTOOLS_COMMANDS.intersection(sys.argv):
    import setuptools

    extra_setuptools_args = dict(
        zip_safe=False,  # the package can run out of an .egg file
        include_package_data=True,
        extras_require={
            'alldeps': (
                'numpy >= {0}'.format(NUMPY_MIN_VERSION),
                'scipy >= {0}'.format(SCIPY_MIN_VERSION),
            ),
        },
    )
else:
    extra_setuptools_args = dict()


class CleanCommand(Clean):
    description = ("Remove build directories, and compiled file in the "
                   "source tree")

    def run(self):
        Clean.run(self)
        if os.path.exists('build'):
            shutil.rmtree('build')
        for dirpath, dirnames, filenames in os.walk('pastis'):
            for filename in filenames:
                if (filename.endswith('.so') or filename.endswith('.pyd')
                   or filename.endswith('.dll')
                   or filename.endswith('.pyc')):
                    os.unlink(os.path.join(dirpath, filename))


def configuration(parent_package='', top_path=None):
    if os.path.exists('MANIFEST'):
        os.remove('MANIFEST')

    from numpy.distutils.misc_util import Configuration
    config = Configuration(None, parent_package, top_path)

    config.add_subpackage('pastis')

    return config


def setup_package():
    metadata = dict(
        configuration=configuration,
        name=DISTNAME,
        maintainer=MAINTAINER,
        maintainer_email=MAINTAINER_EMAIL,
        description=DESCRIPTION,
        version=VERSION,
        scripts=["pastis/script/pastis-mds",
                 'pastis/script/pastis-nmds',
                 "pastis/script/pastis-pm1",
                 "pastis/script/pastis-pm2"],
        classifiers=[
            'Intended Audience :: Science/Research',
            'Intended Audience :: Developers',
            'License :: OSI Approved',
            'Programming Language :: C',
            'Programming Language :: Python',
            'Topic :: Software Development',
            'Topic :: Scientific/Engineering',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: POSIX',
            'Operating System :: Unix',
            'Operating System :: MacOS'],
        **extra_setuptools_args)

    if len(sys.argv) == 1 or (
            len(sys.argv) >= 2 and ('--help' in sys.argv[1:] or
                                    sys.argv[1] in ('--help-commands',
                                                    'egg_info',
                                                    '--version',
                                                    'clean'))):
        # For these actions, NumPy is not required
        #
        # They are required to succeed without Numpy for example when
        # pip is used to install Scikit-learn when Numpy is not yet present in
        # the system.
        try:
            from setuptools import setup
        except ImportError:
            from distutils.core import setup

        metadata['version'] = VERSION
    else:
        from numpy.distutils.core import setup

        metadata['configuration'] = configuration

    setup(**metadata)


if __name__ == "__main__":
    setup_package()



import numpy as np
from .validation import is_symetric_or_tri


def get_intra_mask(lengths):
    """
    Returns a mask for intrachromosomal interactions

    Parameters
    ----------
    lengths : ndarray, (n, )
        lengths of the chromosomes

    Returns
    -------
    mask : ndarray (m, m)
        boolean mask
    """
    # Copy the lengths in order not to modify the original matrix
    mask = np.zeros((lengths.sum(), lengths.sum()))
    begin = 0
    for end in lengths.cumsum():
        mask[begin:end, begin:end] = 1
        begin = end
    return mask.astype(bool)


def get_inter_mask(lengths):
    """
    Returns a mask for interchromosomal interactions

    Parameters
    ----------
    lengths : ndarray, (n, )
        lengths of the chromosomes

    Returns
    -------
    mask : ndarray of dtype boolean
        boolean mask
    """
    intra_mask = get_intra_mask(lengths)
    return np.invert(intra_mask)


def get_genomic_distances(lengths):
    """
    Returns a matrix of the genomic distances

    Inter chromosomal interactions are set to -1

    Parameters
    ----------
    lengths : ndarray (n, )
        lengths of the chromosomes

    Returns
    -------
    dis: ndarray (n, n), dtype: int
        returns the genomic distance matrix, with -1 for inter chromosomal
        interactions
    """
    inter_mask = get_inter_mask(lengths)
    n = lengths.sum()

    dis = np.concatenate([np.concatenate([np.arange(i, 0, -1),
                                          np.arange(n - i)])
                          for i in range(n)])
    dis = dis.reshape((n, n))
    dis[inter_mask] = -1

    return dis.astype(int)


def extract_sub_contact_map(counts, lengths, chromosomes):
    """
    Extract the contact map associated to a given list of chromosome

    Parameters
    ----------
    counts : ndarray (n, n)

    lengths : ndarray (L, )

    chromosomes : list of ids

    Returns
    -------
    sub_counts, sub_lengths : (ndarray, ndarray)

    Examples
    --------

    >>> from iced import datasets
    >>> from iced.utils import extract_sub_contact_map
    >>> counts, lengths = datasets.load_sample_yeast()
    >>> scounts, slengths = extract_sub_contact_map(counts, lengths, [0, 2])
    >>> print len(counts), len(scounts)
    ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
    350 56
    """
    chromosomes = np.array(chromosomes)
    if chromosomes.max() >= len(lengths):
        raise ValueError(
            "The chromosomes provided are not compatible with the "
            "lengths array. Possible values are"
            " %s" % " ".join("%s" % i for i in np.arange(len(lengths))))
    if lengths.sum() != counts.shape[0]:
        raise ValueError(
            "The lengths provided is incompatible with the counts matrix"
            "shape. The total lengths is %d while the contact count matrix "
            "is %d" % (lengths.sum(), counts.shape[0]))

    is_symetric_or_tri(counts)
    chromosomes.sort()

    new_lengths = lengths[chromosomes]
    new_counts = np.zeros((new_lengths.sum(), new_lengths.sum()))
    begin1, end1 = 0, 0
    for i, l1 in enumerate(lengths):
        end1 += l1
        if i not in chromosomes:
            begin1 = end1
            continue
        # Find position of this pair of chromosome in the matrix
        new_num_chrom = (chromosomes == i).argmax()
        if new_num_chrom == 0:
            new_begin1 = 0
        else:
            new_begin1 = new_lengths.cumsum()[new_num_chrom - 1]
        new_end1 = new_lengths.cumsum()[new_num_chrom]

        begin2, end2 = 0, 0
        for j, l2 in enumerate(lengths):
            end2 += l2
            if j not in chromosomes:
                begin2 = end2
                continue
            # Find position of this pair of chromosome in the matrix
            new_num_chrom = (chromosomes == j).argmax()
            if new_num_chrom == 0:
                new_begin2 = 0
            else:
                new_begin2 = new_lengths.cumsum()[new_num_chrom - 1]
            new_end2 = new_lengths.cumsum()[new_num_chrom]
            new_counts[new_begin1:new_end1,
                       new_begin2:new_end2] = counts[begin1:end1, begin2:end2]
            begin2 = end2

        begin1 = end1

    return new_counts, new_lengths


def undersample_per_chr(X, lengths):
    """
    Undersample matrix to chromosomes

    Undersample the matrix ununiformaly per chromosome lengths.

    Parameters
    ----------
    X : ndarray (n, n)
        The matrix to undersample

    lengths : ndarray (L, )
        Lengths of the chromosomes

    Returns
    -------
    undersampled_X : ndarray (L, L)
        `X` undersampled per chromosome
    """
    lengths_cum = lengths.cumsum()
    chr1_begin = 0
    undersampled_X = np.zeros((len(lengths), len(lengths)))
    for i, chr1_end in enumerate(lengths_cum):
        chr2_begin = 0
        for j, chr2_end in enumerate(lengths_cum):
            surface = X[chr1_begin:chr1_end, chr2_begin:chr2_end]
            undersampled_X[i, j] = surface[np.invert(np.isnan(surface))].mean()
            chr2_begin = chr2_end
        chr1_begin = chr1_end

    return undersampled_X


def downsample_resolution(counts, lengths, factor=2):
    """
    Downsamples the resolution of a matrix

    Parameters
    ----------
    counts : ndarray (N, N)
        contact counts matrix to downsample

    lengths : ndarray (L, )
        chromosomes lengths

    coef : int, optionnal, default: 2
        downsample resolution of the counts matrix by `coef`

    Returns
    -------
    target_counts, target_lengths : ndarray
    """
    if factor == 1:
        return counts, lengths
    # FIXME there is probably a better way to do this
    target_lengths = np.ceil(lengths.astype(float) / factor).astype(int)
    target_counts = np.zeros((target_lengths.sum(),
                              target_lengths.sum()))
    begin_i, end_i = 0, 0
    target_begin_i, target_end_i = 0, 0
    for i, length_i in enumerate(lengths):
        end_i += length_i
        target_end_i += target_lengths[i]
        begin_j, end_j = 0, 0
        target_begin_j, target_end_j = 0, 0
        for j, length_j in enumerate(lengths):
            end_j += length_j
            target_end_j += target_lengths[j]

            sub_counts = counts[begin_i:end_i, begin_j:end_j]
            sub_target_counts = target_counts[target_begin_i:target_end_i,
                                              target_begin_j:target_end_j]
            d = np.zeros(sub_target_counts.shape)
            for start in range(factor):
                s = sub_counts[start::factor, start::factor]
                d[:s.shape[0], :s.shape[1]] += np.invert(np.isnan(s))
                s[np.isnan(s)] = 0
                sub_target_counts[:s.shape[0], :s.shape[1]] += s
            sub_target_counts /= d

            begin_j = end_j
            target_begin_j = target_end_j
        begin_i = end_i
        target_begin_i = target_end_i
    return target_counts, target_lengths


def _change_lengths_resolution(lengths, resolution=10000, copy=True):
    if copy:
        lengths = lengths.copy()
    lengths = lengths.astype(float)
    return np.ceil(lengths / resolution).astype(int)


def get_chromosome_counts(counts, lengths, chromosome):
    """
    """
    if chromosome > len(lengths) - 1:
        raise ValueError("Chromosome %d doesn't exists. Possible values are "
                         "from 0 to %d" % (chromosome, len(lengths) - 1))

    if len(counts) != lengths.sum():
        raise ValueError("The total lengths and the counts matrix shape "
                         "should be the same. They are respectively %d and %d"
                         % (lengths.sum(), len(counts)))

    lengths_cum = np.concatenate([[0], lengths.cumsum()])
    return counts[lengths_cum[chromosome]:lengths_cum[chromosome + 1],
                  lengths_cum[chromosome]:lengths_cum[chromosome + 1]]



import numpy as np
from scipy import sparse
from ._normalization_ import _update_normalization_csr
from .utils import is_symetric_or_tri, is_tri


def ICE_normalization(X, SS=None, max_iter=3000, eps=1e-4, copy=True,
                      norm='l1', verbose=0, output_bias=False):
    """
    ICE normalization

    The imakaev normalization of Hi-C data consists of iteratively estimating
    the bias such that all the rows and columns (ie loci) have equal
    visibility.

    Parameters
    ----------
    X : ndarray or sparse array (n, n)
        raw interaction frequency matrix

    max_iter : integer, optional, default: 3000
        Maximum number of iteration

    eps : float, optional, default: 1e-4
        the relative increment in the results before declaring convergence.

    copy : boolean, optional, default: True
        If copy is True, the original data is not modified.

    norm : string, optional, default: l1
        If set to "l1", will compute the ICE algorithm of the paper. Else, the
        algorithm is adapted to use the l2 norm, as suggested in the SCN
        paper.

    output_bias : boolean, optional, default: False
        whether to output the bias vector.

    Returns
    -------
    X, (bias) : ndarray (n, n)
        Normalized IF matrix and bias of output_bias is True

    Example
    -------
    .. plot:: examples/normalization/plot_ICE_normalization.py
    """
    if copy:
        X = X.copy()

    if sparse.issparse(X):
        if not sparse.isspmatrix_csr(X):
            X = sparse.csr_matrix(X, dtype="float")
        X.sort_indices()
    else:
        X[np.isnan(X)] = 0
    X = X.astype('float')

    mean = X.mean()
    m = X.shape[0]
    is_symetric_or_tri(X)
    old_dbias = None
    bias = np.ones((m, 1))
    for it in np.arange(max_iter):
        if norm == 'l1':
            # Actually, this should be done if the matrix is diag sup or diag
            # inf
            if is_tri(X):
                sum_ds = X.sum(axis=0) + X.sum(axis=1).T - X.diagonal()
            else:
                sum_ds = X.sum(axis=0)
        elif norm == 'l2':
            if is_tri(X):
                sum_ds = ((X**2).sum(axis=0) +
                          (X**2).sum(axis=1).T -
                          (X**2).diagonal())
            else:
                sum_ds = (X**2).sum(axis=0)

        if SS is not None:
            raise NotImplementedError

        dbias = sum_ds.reshape((m, 1))
        # To avoid numerical instabilities
        dbias /= dbias[dbias != 0].mean()

        dbias[dbias == 0] = 1
        bias *= dbias

        if sparse.issparse(X):
            X = _update_normalization_csr(X, np.array(dbias).flatten())
        else:
            X /= dbias * dbias.T

        X *= mean / X.mean()
        if old_dbias is not None and np.abs(old_dbias - dbias).sum() < eps:
            if verbose > 1:
                print("break at iteration %d" % (it,))
            break

        if verbose > 1 and old_dbias is not None:
            print('ICE at iteration %d %s' %
                  (it, np.abs(old_dbias - dbias).sum()))

        # Rescaling X so that the  mean always stays the same.
        # XXX should probably do this properly, ie rescale the bias such that
        # the total scaling of the contact counts don't change.
        old_dbias = dbias.copy()
    if output_bias:
        return X, bias
    else:
        return X


def SCN_normalization(X, max_iter=300, eps=1e-6, copy=True):
    """
    Sequential Component Normalization

    Parameters
    ----------
    X : ndarray (n, n)
        raw interaction frequency matrix

    max_iter : integer, optional, default: 300
        Maximum number of iteration

    eps : float, optional, default: 1e-6
        the relative increment in the results before declaring convergence.

    copy : boolean, optional, default: True
        If copy is True, the original data is not modified.

    Returns
    -------
    X : ndarray,
        Normalized IF
    """
    # X needs to be square, else it's gonna fail

    m, n = X.shape
    if m != n:
        raise ValueError

    if copy:
        X = X.copy()
    X = X.astype(float)

    for it in np.arange(max_iter):
        sum_X = np.sqrt((X ** 2).sum(0))
        sum_X[sum_X == 0] = 1
        X /= sum_X
        X = X.T
        sum_X = np.sqrt((X ** 2).sum(0))
        sum_X[sum_X == 0] = 1
        X /= sum_X
        X = X.T

        if np.abs(X - X.T).sum() < eps:
            print("break at iteration %d" % (it,))
            break

    return X



import os
import shutil
import stat
import subprocess
import numpy as np

from sklearn.metrics import euclidean_distances
from sklearn.isotonic import IsotonicRegression

from . import write
from .config import parse


max_iter = 5

CMD_PM = ('%s -o %s -w 8 '
          '-r %d '
          '-k %s '
          '-i %s '
          '-c %s -y 1 -a %f -b %f > %s')


CMD_MDS = ('%s -o %s -w 8 '
           '-r %d '
           '-k %s '
           '-i %s '
           '-c %s -y 1 > %s')


def run_mds(directory):
    print "Running MDS"

    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)

    random_state = np.random.RandomState(seed=options["seed"])

    # First, compute MDS
    counts = np.load(os.path.join(directory,
                                  options["counts"]))
    wish_distances = compute_wish_distances(counts, alpha=options["alpha"],
                                            beta=options["beta"])
    wish_distances *= 1000
    wish_distances[np.isinf(wish_distances) | np.isnan(wish_distances)] = 0
    lengths = np.loadtxt(
        os.path.join(directory, options["organism_structure"]))

    try:
        len(lengths)
    except TypeError:
        lengths = np.array([lengths])

    write(wish_distances,
          os.path.join(directory,
                       'wish_distances.txt'),
          lengths=lengths,
          resolution=options["resolution"])

    # Write initial point
    X = 1. - 2. * random_state.rand(len(counts) * 3)
    np.savetxt(os.path.join(directory,
                            "MDS." + options["output_name"] + ".temp.txt"),
               X)

    cmd = CMD_MDS % (options["binary_mds"],
                     os.path.join(directory,
                                  "MDS." + options["output_name"]),
                     options["resolution"],
                     os.path.join(directory,
                                  options["organism_structure"]),
                     os.path.join(directory,
                                  "wish_distances.txt"),
                     options["chromosomes"],
                     os.path.join(directory,
                                  'MDS.log'))

    filename = os.path.join(directory, 'MDS.sh')
    fileptr = open(filename, 'wb')
    fileptr.write(cmd)
    fileptr.close()
    st = os.stat(filename)
    os.chmod(filename, st.st_mode | stat.S_IXUSR)
    subprocess.call(filename.split(), shell='True')
    return True


def mds(wish_distances, X=None, random_state=None):
    """
    Computes the MDS solution

    Parameters
    ----------
    wish_distances : ndarray
        Array of wish distances
    """
    if len(wish_distances.shape) != 2:
        raise ValueError("The wish distances should be a 2D ndarray.")
    if wish_distances.shape[0] == wish_distances.shape[1]:
        raise ValueError(
            "The wish distances should be a squared array (of "
            "shape (n by n).")
    n = wish_distances.shape[0]
    if X is not None and len(X) != n:
        raise ValueError(
            "The length of ")


def compute_wish_distances(counts, alpha=-3, beta=1):
    """
    Computes wish distances from a counts matrix

    Parameters
    ----------
    counts : ndarray
        Interaction counts matrix

    alpha : float, optional, default: -3
        Coefficient of the power law

    beta : float, optional, default: 1
        Scaling factor

    Returns
    -------
    wish_distances
    """
    wish_distances = counts.copy() / beta
    wish_distances[wish_distances != 0] **= 1. / alpha
    # FIXME this doesn't use beta
    return beta * wish_distances


def run_nmds(directory):
    print directory

    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)
    run_mds(directory)

    for i in range(0, max_iter):
        if i == 0:
            try:
                X = np.loadtxt(
                    os.path.join(directory,
                                 "MDS." + options["output_name"] + ".txt"))
            except IOError:
                return
        else:
            X = np.loadtxt(
                os.path.join(directory,
                             '%d.NMDS.' % (i) + options["output_name"] +
                             ".txt"))

        X = X.reshape((len(X) / 3, 3))

        dis = euclidean_distances(X) * 1000
        counts = np.load(
            os.path.join(directory, options["counts"]))
        counts[np.isnan(counts)] = 0

        wish_distances = np.zeros(counts.shape)

        print "Fitting isotonic regression..."
        ir = IsotonicRegression()
        wish_distances[counts != 0] = ir.fit_transform(
            1. / counts[counts != 0],
            dis[counts != 0])
        print "writing wish distances"

        lengths = np.loadtxt(
            os.path.join(directory, options["organism_structure"]))

        try:
            len(lengths)
        except TypeError:
            lengths = np.array([lengths])

        write(wish_distances,
              os.path.join(directory,
                           '%d.NMDS.wish_distances.txt' % i),
              lengths=lengths, resolution=options["resolution"])

        if i == 0:
            shutil.copy(
                os.path.join(directory,
                             "MDS." + options["output_name"] + ".txt"),
                os.path.join(directory,
                             '%d.NMDS.' % (i + 1) + options["output_name"] +
                             ".temp.txt"))
        else:
            shutil.copy(
                os.path.join(directory,
                             '%d.NMDS.' % i + options["output_name"] + ".txt"),
                os.path.join(directory,
                             '%d.NMDS.' % (i + 1) + options["output_name"] +
                             ".temp.txt"))

        cmd = CMD_MDS % (options["binary_mds"],
                         os.path.join(directory,
                                      "%d.NMDS." % (i + 1) +
                                      options["output_name"]),
                         options["resolution"],
                         os.path.join(directory,
                                      options["organism_structure"]),
                         os.path.join(directory,
                                      "%d.NMDS.wish_distances.txt" % (i)),
                         options["chromosomes"],
                         os.path.join(directory,
                                      str(i + 1) + '.NMDS.log'))

        filename = os.path.join(directory, str(i + 1) + '.NMDS.sh')
        fileptr = open(filename, 'wb')
        fileptr.write(cmd)
        fileptr.close()
        st = os.stat(filename)
        os.chmod(filename, st.st_mode | stat.S_IXUSR)
        subprocess.call(filename.split(), shell='True')


def run_pm1(directory):
    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)
    run_mds(directory)
    shutil.copy(
        os.path.join(directory,
                     "MDS." + options["output_name"] + ".txt"),
        os.path.join(directory,
                     'PM1.' + options["output_name"] +
                     ".temp.txt"))

    counts = np.load(os.path.join(directory, options["counts"]))
    lengths = np.loadtxt(
        os.path.join(directory,
                     options["organism_structure"]))

    try:
        len(lengths)
    except TypeError:
        lengths = np.array([lengths])

    write(counts,
          os.path.join(directory,
                       'counts.txt'),
          lengths=lengths,
          resolution=options["resolution"])

    cmd = CMD_PM % (options["binary_pm"],
                    os.path.join(directory,
                                 "PM1." + options["output_name"]),
                    options["resolution"],
                    os.path.join(directory, options["organism_structure"]),
                    os.path.join(directory,
                                 'counts.txt'),
                    options["chromosomes"],
                    options["alpha"],
                    options["beta"],
                    os.path.join(directory,
                                 'PM1.log'))

    filename = os.path.join(directory, 'PM1.sh')
    fileptr = open(filename, 'wb')
    fileptr.write(cmd)
    fileptr.close()
    st = os.stat(filename)
    os.chmod(filename, st.st_mode | stat.S_IXUSR)
    subprocess.call(filename.split(), shell='True')
    return True


def run_pm2(directory):
    from .poisson_model_power_law import estimate_alpha_beta
    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)
    alpha, beta = options["alpha"], options["beta"]
    run_mds(directory)

    # Save counts in a format the C++ code can use
    counts = np.load(
        os.path.join(directory, options["counts"]))
    lengths = np.loadtxt(
        os.path.join(directory,
                     options["organism_structure"]))

    try:
        len(lengths)
    except TypeError:
        lengths = np.array([lengths])

    write(counts,
          os.path.join(directory,
                       'counts.txt'),
          lengths=lengths,
          resolution=options["resolution"])

    for i in range(0, max_iter):
        if i == 0:
            try:
                X = np.loadtxt(
                    os.path.join(directory,
                                 "MDS." + options["output_name"] + ".txt"))
            except IOError:
                return
        else:
            X = np.loadtxt(
                os.path.join(directory,
                             '%d.PM2.' % (i) + options["output_name"] +
                             ".txt"))

        X = X.reshape((len(X) / 3, 3))

        # Fit alpha and beta
        alpha, beta = estimate_alpha_beta(counts, X, ini=np.array([alpha]))

        if i == 0:
            shutil.copy(
                os.path.join(directory,
                             "MDS." + options["output_name"] + ".txt"),
                os.path.join(directory,
                             '%d.PM2.' % (i + 1) + options["output_name"] +
                             ".temp.txt"))
        else:
            shutil.copy(
                os.path.join(directory,
                             '%d.PM2.' % i + options["output_name"] + ".txt"),
                os.path.join(directory,
                             '%d.PM2.' % (i + 1) + options["output_name"] +
                             ".temp.txt"))

        cmd = CMD_PM % (options["binary_pm"],
                        os.path.join(directory,
                                     "%d.PM2." % (i + 1) +
                                     options["output_name"]),
                        options["resolution"],
                        os.path.join(directory,
                                     options["organism_structure"]),
                        os.path.join(directory,
                                     'counts.txt'),
                        options["chromosomes"],
                        alpha,
                        beta,
                        os.path.join(directory,
                                     str(i + 1) + '.PM2.log'))

        filename = os.path.join(directory, str(i + 1) + '.PM2.sh')
        fileptr = open(filename, 'wb')
        fileptr.write(cmd)
        fileptr.close()
        st = os.stat(filename)
        os.chmod(filename, st.st_mode | stat.S_IXUSR)
        subprocess.call(filename.split(), shell='True')



import numpy as np


def get_intra_mask(lengths, resolution=10000):
    """
    Returns a mask for intrachromosomal interactions

    Parameters
    ----------
    lengths : ndarray, (n, )
        lengths of the chromosomes

    resolution : int, optional, default: 10000
        resolution in which the mask should be created

    Returns
    -------
    mask : ndarray (m, m)
        boolean mask
    """
    # Copy the lengths in order not to modify the original matrix
    lengths = _change_lengths_resolution(lengths, resolution=resolution)
    mask = np.zeros((lengths.sum(), lengths.sum()))
    begin = 0
    for end in lengths.cumsum():
        mask[begin:end, begin:end] = 1
        begin = end
    return mask.astype(bool)


def get_inter_mask(lengths, resolution=10000):
    """
    Returns a mask for interchromosomal interactions

    Parameters
    ----------
    lengths : ndarray, (n, )
        lengths of the chromosomes

    resolution : int, optional, default: 10000
        resolution in which the mask should be created

    Returns
    -------
    mask : ndarray of dtype boolean
        boolean mask
    """
    intra_mask = get_intra_mask(lengths, resolution=resolution)
    return np.invert(intra_mask)


def get_genomic_distances(lengths, resolution=10000):
    """
    Returns a matrix of the genomic distances

    Inter chromosomal interactions are set to -1

    Parameters
    ----------
    lengths : ndarray (n, )
        lengths of the chromosomes

    resolution : int, optional, default: 10000
        resolution in which to compute the matrix

    Returns
    -------
    dis: ndarray (n, n), dtype: int
        returns the genomic distance matrix, with -1 for inter chromosomal
        interactions
    """
    inter_mask = get_inter_mask(lengths, resolution=resolution)
    lengths = _change_lengths_resolution(lengths, resolution=resolution)
    n = lengths.sum()

    dis = np.concatenate([np.concatenate([np.arange(i, 0, -1),
                                          np.arange(n - i)])
                          for i in range(n)])
    dis = dis.reshape((n, n))
    dis[inter_mask] = -1

    return dis.astype(int)


def undersample_per_chr(X, lengths, resolution=10000):
    """
    Undersample matrix to chromosomes

    Undersample the matrix ununiformaly per chromosome lengths.

    Parameters
    ----------
    X : ndarray (n, n)
        The matrix to undersample

    lengths : ndarray (L, )
        Lengths of the chromosomes

    resolution : integer, optional, default: 10000
        Resolution in which the matrix `X` is provided

    Returns
    -------
    undersampled_X : ndarray (L, L)
        `X` undersampled per chromosome
    """
    lengths = _change_lengths_resolution(lengths, resolution=resolution)
    lengths_cum = lengths.cumsum()
    chr1_begin = 0
    undersampled_X = np.zeros((len(lengths), len(lengths)))
    for i, chr1_end in enumerate(lengths_cum):
        chr2_begin = 0
        for j, chr2_end in enumerate(lengths_cum):
            surface = X[chr1_begin:chr1_end, chr2_begin:chr2_end]
            undersampled_X[i, j] = surface[np.invert(np.isnan(surface))].mean()
            chr2_begin = chr2_end
        chr1_begin = chr1_end

    return undersampled_X


def _change_lengths_resolution(lengths, resolution=10000, copy=True):
    if copy:
        lengths = lengths.copy()
    lengths = lengths.astype(float)
    return np.ceil(lengths / resolution).astype(int)


def get_chromosome_counts(counts, lengths, chromosome):
    """
    """
    if chromosome > len(lengths) - 1:
        raise ValueError("Chromosome %d doesn't exists. Possible values are "
                         "from 0 to %d" % (chromosome, len(lengths) - 1))

    if len(counts) != lengths.sum():
        raise ValueError("The total lengths and the counts matrix shape "
                         "should be the same. They are respectively %d and %d"
                         % (lengths.sum(), len(counts)))

    lengths_cum = np.concatenate([[0], lengths.cumsum()])
    return counts[lengths_cum[chromosome]:lengths_cum[chromosome + 1],
                  lengths_cum[chromosome]:lengths_cum[chromosome + 1]]
